/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.util.Map;
import java.util.HashMap;
import java.util.TreeMap;

/**
 * Demonstrates the {@link RandomMapCache} class.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class RandomCacheDemo {
   private static int getSetSize = 32;
   private static String[] keys = new String[1280];
   private static Map hashMap = new HashMap();
   private static Map treeMap = new TreeMap();
   private static RandomMapCache cachedHashMap = new RandomMapCache(hashMap, 128);
   private static RandomMapCache cachedTreeMap = new RandomMapCache(treeMap, 128);
   public static void main(String[] args) {
      fillMaps();
      int iterations = 100000;
      long start;

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readCachedHashMap();
      System.out.println("readCachedHashMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readCachedHashMap();
      System.out.println("readCachedHashMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readHashMap();
      System.out.println("readHashMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readHashMap();
      System.out.println("readHashMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readTreeMap();
      System.out.println("readTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readTreeMap();
      System.out.println("readTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readCachedTreeMap();
      System.out.println("readCachedTreeMap() " + (System.currentTimeMillis()-start));

      start = System.currentTimeMillis();
      for (int i=0; i<iterations; i++) readCachedTreeMap();
      System.out.println("readCachedTreeMap() " + (System.currentTimeMillis()-start));

      System.out.println("cachedHashMap.getHitRatio() " + cachedHashMap.getHitRatio());
      System.out.println("cachedTreeMap.getHitRatio() " + cachedTreeMap.getHitRatio());
   }

   public static void readCachedHashMap() {
      for (int i=0; i<getSetSize; i++) {
         cachedHashMap.get(keys[i]);
      }
   }

   public static void readCachedTreeMap() {
      for (int i=0; i<getSetSize; i++) {
         cachedTreeMap.get(keys[i]);
      }
   }

   public static void readHashMap() {
      for (int i=0; i<getSetSize; i++) {
         hashMap.get(keys[i]);
      }
   }

   public static void readTreeMap() {
      for (int i=0; i<getSetSize; i++) {
         treeMap.get(keys[i]);
      }
   }

   private static void fillMaps() {
      // fill
      for (int i=0; i<keys.length; i++) {
         keys[i] = "key" + i;
         cachedHashMap.put(keys[i], "value");
         cachedTreeMap.put(keys[i], "value");
      }
   }


}
