/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.util.Map;
import java.util.TreeMap;
import java.util.Random;

/**
 * Compares {@link SkipList} with {@link TreeMap}. TreeMap usually wins...
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class SkipListDemo {
   private static String[] elements = new String[500000];
   public static void main(String[] args) {
      for (int i=0; i<elements.length; i++) elements[i] = "key " + i;
      System.gc();
      testGet();
      System.gc();
      testRemove();
      System.gc();
      shuffle(elements);
      System.gc();
      testPut();
      System.gc();
      testGet();
      System.gc();
      testRemove();
   }

   private static void testGet() {
      long start;
      long time;
      Map map;

      map = new SkipList(elements.length);
      put(map);
      start = System.currentTimeMillis();
      get(map);
      time = System.currentTimeMillis() - start;
      System.out.println("get(SkipList) first run: " + time);

      map = new SkipList(elements.length);
      put(map);
      start = System.currentTimeMillis();
      get(map);
      time = System.currentTimeMillis() - start;
      System.out.println("get(SkipList) second run: " + time);

      map = new TreeMap();
      put(map);
      start = System.currentTimeMillis();
      get(map);
      time = System.currentTimeMillis() - start;
      System.out.println("get(TreeMap) first run: " + time);

      map = new TreeMap();
      put(map);
      start = System.currentTimeMillis();
      get(map);
      time = System.currentTimeMillis() - start;
      System.out.println("get(TreeMap) second run: " + time);
   }


   private static void testPut() {
      long start;
      long time;
      Map map;

      map = new SkipList(elements.length);
      start = System.currentTimeMillis();
      put(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(SkipList) first run: " + time);

      map = new SkipList(elements.length);
      start = System.currentTimeMillis();
      put(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(SkipList) second run: " + time);

      map = new TreeMap();
      start = System.currentTimeMillis();
      put(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(TreeMap) first run: " + time);

      map = new TreeMap();
      start = System.currentTimeMillis();
      put(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(TreeMap) second run: " + time);
   }

   private static void testRemove() {
      long start;
      long time;
      Map map;

      map = new SkipList(elements.length);
      put(map);
      start = System.currentTimeMillis();
      remove(map);
      time = System.currentTimeMillis() - start;
      System.out.println("remove(SkipList) first run: " + time);

      map = new SkipList(elements.length);
      put(map);
      start = System.currentTimeMillis();
      remove(map);
      time = System.currentTimeMillis() - start;
      System.out.println("remove(SkipList) second run: " + time);

      map = new TreeMap();
      put(map);
      start = System.currentTimeMillis();
      remove(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(TreeMap) first run: " + time);

      map = new TreeMap();
      put(map);
      start = System.currentTimeMillis();
      remove(map);
      time = System.currentTimeMillis() - start;
      System.out.println("put(TreeMap) second run: " + time);
   }

   private static void put(Map map) {
      for (int i=0; i<elements.length; i++) {
         map.put(elements[i], "value");
      }
   }

   private static void get(Map map) {
      for (int i=0; i<elements.length; i++) {
         map.get(elements[i]);
      }
   }

   private static void remove(Map map) {
      for (int i=0; i<elements.length; i++) {
         map.remove(elements[i]);
      }
   }
   private static Random rnd = new Random();

   // Methods for shuffeling arrays
   private static void shuffle(Object[] array) {
      // Shuffle array
      for (int i=array.length; i>1; i--)
          swap(array, i-1, rnd.nextInt(i));
   }

   private static void swap(Object[] arr, int i, int j) {
       Object tmp = arr[i];
       arr[i] = arr[j];
       arr[j] = tmp;
   }

}
