/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.httpd;

import java.net.Socket;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

/**
 * Acceptors call the accept() method of a {@link ServerSocketChannel},
 * instanciate a {@link Connection} objekt with the accepted connection
 * and register it with a {@link ConnectionSelector}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
class Acceptor extends Thread {

   private static int _no; // instance counter
   private ServerSocketChannel serverSocketChannel;
   private ConnectionSelector connectionSelector;

   /**
    * Starts this Acceptor thread.
    */
   public Acceptor(ServerSocketChannel serverSocketChannel, ConnectionSelector connectionSelector) {
      super("Acceptor " + (_no++));
      this.serverSocketChannel = serverSocketChannel;
      this.connectionSelector = connectionSelector;
      start();
   }

   /**
    * Accepts connections and registers them with
    * {@link ConnectionSelector#enqueue(Connection)}.
    */
   public void run() {
      while (true) {
         SocketChannel socketChannel = null;
         try {
            socketChannel = serverSocketChannel.accept();
            connectionSelector.enqueue(new Connection(socketChannel));
         } catch (Exception e) {
            e.printStackTrace();
            // clean up, if necessary
            if (socketChannel != null) {
               try {
                  socketChannel.close();
               } catch (Exception ee) {
                  // ignore
               }
            }
         }
      }
   }
}
