/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

import java.util.List;
import java.util.ArrayList;
import java.lang.IndexOutOfBoundsException;

/**
 * Demonstrates exceptions terminated loops. Output in CSV.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ExceptionTerminatedLoopDemo2 {
   private static List list = new ArrayList();

   public static void main(String[] args) {
      System.out.println("Elements,normalLoop 1st,normalLoop 2nd,normalLoop 3rd,exceptionTerminatedLoop 1st,exceptionTerminatedLoop 2nd,exceptionTerminatedLoop3rd");
      for (int elementsInList = 500; elementsInList<5500; elementsInList+=500) {
         runTests(elementsInList, 10000);
      }
   }

   private static void runTests(int elementsInList, int iterations) {
      System.out.print(elementsInList+",");
      long start;
      long time;

      // fill list
      list.clear();
      System.gc();
      for (int i=0; i<elementsInList; i++) list.add("string");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalLoop();
      time = System.currentTimeMillis() - start;
      System.out.print(time + ",");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalLoop();
      time = System.currentTimeMillis() - start;
      System.out.print(time + ",");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) normalLoop();
      time = System.currentTimeMillis() - start;
      System.out.print(time + ",");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) exceptionTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.print(time + ",");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) exceptionTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.print(time + ",");

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) exceptionTerminatedLoop();
      time = System.currentTimeMillis() - start;
      System.out.println(time);
   }

   private static String exceptionTerminatedLoop() {
      String s = null;
      try {
         for (int i=0; true; i++) {
            s = (String)list.get(i);
         }
      } catch (IndexOutOfBoundsException e) {
      }
      return s;
   }

   private static String normalLoop() {
      String s = null;
      for (int i=0, n = list.size(); i<n; i++) {
         s = (String)list.get(i);
      }
      return s;
   }

}
