/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.ifsloopsandswitches;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.lang.IndexOutOfBoundsException;

/**
 * Demonstrates several ways of iterating over a list.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class ListIteratorLoopDemo {
   private static List arrayList = new ArrayList();
   private static List linkedList = new LinkedList();

   public static void main(String[] args) {
      int iterations = 10000;
      long start;
      long time;

      // fill lists
      for (int i=0; i<1000; i++) arrayList.add("string");
      linkedList.addAll(arrayList);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) arrayListLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("arrayListLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) arrayListLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("arrayListLoop() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) arrayListIteratorLoop();
       time = System.currentTimeMillis() - start;
      System.out.println("arrayListIteratorLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) arrayListIteratorLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("arrayListIteratorLoop() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) linkedListLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("linkedListLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) linkedListLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("linkedListLoop() second run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) linkedListIteratorLoop();
       time = System.currentTimeMillis() - start;
      System.out.println("linkedListIteratorLoop() first run: " + time);

      start = System.currentTimeMillis();
      for (int i = 0; i < iterations; i++) linkedListIteratorLoop();
      time = System.currentTimeMillis() - start;
      System.out.println("linkedListIteratorLoop() second run: " + time);
   }

   private static String arrayListIteratorLoop() {
      String s = null;
      for (Iterator i = arrayList.iterator(); i.hasNext();) {
         s = (String)i.next();
      }
      return s;
   }

   private static String linkedListIteratorLoop() {
      String s = null;
      for (Iterator i = linkedList.iterator(); i.hasNext();) {
         s = (String)i.next();
      }
      return s;
   }

   private static String arrayListLoop() {
      String s = null;
      for (int i=0, n = arrayList.size(); i<n; i++) {
         s = (String)arrayList.get(i);
      }
      return s;
   }

   private static String linkedListLoop() {
      String s = null;
      for (int i=0, n = linkedList.size(); i<n; i++) {
         s = (String)linkedList.get(i);
      }
      return s;
   }
}
