/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.net.URL;
import java.util.Random;

/**
 * Demonstrates the costs of finding out file metadata.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class LengthAndLastModifiedDemo {
   private static byte[] buf = new byte[1024 * 8];

   public static void main(String[] args) throws Exception {
      new Random().nextBytes(buf);
      File[] file= new File[12];
      for (int i=1; i<file.length+1; i++) {
         file[i-1] = makeFile(i);
      }
      for (int no=5; no<file.length; no++) {
         System.out.println("size " + (file[no].length()/1024) + "kb");
         long start = System.currentTimeMillis();
         for (int i=0; i<10000; i++) {
            readFile(file[no]);
         }
         System.out.println("readFile: " + (System.currentTimeMillis()-start));

         start = System.currentTimeMillis();
         for (int i=0; i<10000; i++) {
            getLengthAndLastModified(file[no]);
         }
         System.out.println("length/modified: " + (System.currentTimeMillis()-start));
      }
   }

   public static void getLengthAndLastModified(File file) throws IOException {
      file.length();
      file.lastModified();
   }

   public static void readFile(File file) throws IOException {
      FileInputStream in = new FileInputStream(file);
      while (in.read(buf) != -1) {}
      in.close();
   }

   private static File makeFile(int sizeInKB) throws IOException {
      System.out.println("Writing file with size " + (sizeInKB * 1024 * 8));
      File file = File.createTempFile("ResourceCacheDemo", ".tmp");
      FileOutputStream out = new FileOutputStream(file);
      for (int i=0; i<sizeInKB; i++) out.write(buf);
      out.close();
      return file;
   }

}
