/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.memory;

/**
 * This class demonstrates that many VMs don't collect unreferenced objects
 * until the current method has been left.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class NoGCInMethodDemo {

    /**
     * Repeteadly allocates an array using {@link #allocateInMethods(int)}.
     */
    public void allocateInMethods(int size) throws InterruptedException {
        allocateByteArray(size);
        allocateByteArray(size);
        allocateByteArray(size);
        allocateByteArray(size);
        allocateByteArray(size);
        System.out.println();
    }

    private void allocateByteArray(int size) throws InterruptedException {
        System.out.print(".");
        byte[] a = new byte[size];
    }

    /**
     * Repeteadly allocates an array with the given size using
     * a block.
     */
    public void allocate(int size) throws InterruptedException {
        {
            System.out.print("*");
            byte[] a = new byte[size];
        }
        {
            System.out.print("*");
            byte[] a = new byte[size];
        }
        {
            System.out.print("*");
            byte[] a = new byte[size];
        }
        {
            System.out.print("*");
            byte[] a = new byte[size];
        }
        {
            System.out.print("*");
            byte[] a = new byte[size];
        }
    }

    public static void main(String[] args) throws InterruptedException {
        NoGCInMethodDemo i = new NoGCInMethodDemo();
        int size = 50000000; // 50 mb
        i.allocateInMethods(size);
        i.allocate(size);
    }

}
