/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.memviewer;

import java.lang.reflect.Method;
import java.util.Properties;
import java.util.StringTokenizer;

/**
 * Memviewer.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class MemViewer extends Thread {

   private Visualizer visualizer;
   private long interval = 500;

   public MemViewer(String arg) throws Exception {
      StringTokenizer st = new StringTokenizer(arg, ":");
      String[] args = new String[st.countTokens() - 1];
      String type = st.nextToken();
      for (int i = 0; st.hasMoreTokens(); i++) {
         args[i] = st.nextToken();
      }
      Class visualizerClass = getVisualizerClass(type);
      visualizer = (Visualizer) visualizerClass.newInstance();
      visualizer.init(getProperties(args));
   }

   public synchronized void run() {
      while (true) {
         visualizer.showMemory(new MemState());
         try {
            wait(interval);
         } catch (InterruptedException ie) {
            ie.printStackTrace();
         }
      }
   }

   /**
    * Starts the memviewer and calls the main method of the given class.
    */
   public static void main(String[] args) throws Exception {
      if (args.length < 2) usage();
      MemViewer memViewer = new MemViewer(args[0]);
      memViewer.setDaemon(true);
      memViewer.start();
      Method mainMethod = Class.forName(args[1]).getMethod("main", new Class[]{String[].class});
      String[] newArgs = new String[args.length - 1];
      System.arraycopy(args, 1, newArgs, 0, newArgs.length);
      mainMethod.invoke(null, new Object[]{newArgs});
   }

   public static void usage() {
      System.out.println("MemViewer");
      System.out.println("java com.tagtraum.perf.memviewer.MemViewer text[:file=<filename>]|gui|<visualizer classname> <main classname>");
      System.exit(0);
   }

   private static Properties getProperties(String[] args) {
      Properties properties = new Properties();
      for (int i = 0; i < args.length; i++) {
         int index = args[i].indexOf('=');
         if (index != -1 && index < args[i].length() - 1) {
            properties.setProperty(args[i].substring(0, index).toLowerCase(), args[i].substring(index + 1));
         }
      }
      return properties;
   }

   private static Class getVisualizerClass(String type) throws ClassNotFoundException {
      Class visualizerClass;
      if (type.equals("text")) {
         visualizerClass = TextualVisualizer.class;
      } else if (type.equals("gui")) {
         visualizerClass = AWTVisualizer.class;
      } else {
         visualizerClass = Class.forName(type);
      }
      return visualizerClass;
   }

}
