/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.monitor;

/**
 * Classical Deadlock. Run this code with the flag
 * <code>-Xrunhprof:monitor=y</code> and look at the
 * resulting file <code>java.hprof.txt</code>.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class DeadlockDemo extends Thread {

   private static class Lock extends Object {
   };

   private Lock firstLock;
   private Lock secondLock;

   public DeadlockDemo(String name, Lock firstLock, Lock secondLock) {
      super(name);
      this.firstLock = firstLock;
      this.secondLock = secondLock;
   }

   public void run() {
      while (true) {
         synchronized (firstLock) {
            System.out.println(this.getName() + ": Obtained first lock.");
            synchronized (secondLock) {
               System.out.println(this.getName() + ": Obtained second lock.");
            }
         }
      }
   }

   public static void main(String[] args) {
      Lock leftLock = new Lock();
      Lock rightLock = new Lock();
      DeadlockDemo leftRight = new DeadlockDemo("left-right Thread", leftLock, rightLock);
      DeadlockDemo rightLeft = new DeadlockDemo("right-left Thread", rightLock, leftLock);
      leftRight.start();
      rightLeft.start();
   }
}
