/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.serialization;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Locale;

/**
 * Demonstrates serialization differences between the different InternationalDate classes.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 * @see InternationDate1
 * @see InternationDate2
 * @see InternationDate3
 * @see InternationDate4
 * @see InternationDate5
 */
public class InternationalDateDemo {

   // the number of available locales is different in different environments
   // we therefore limit it to a value of 145
   public static final int availableLocales = 145;

   public static void main(String[] args) throws Exception {
      testDate(new InternationalDate1());
      testDate(new InternationalDate2());
      testDate(new InternationalDate3());
      testDate(new InternationalDate4());
      testDate(InternationalDate5.getInstance());
   }

   private static void testDate(Object date) throws Exception {
      double iterations = 2;
      long saveTime = 0;
      long loadTime = 0;
      int size = 0;
      for (int i = 0; i < iterations; i++) {
         ByteArrayOutputStream bout = new ByteArrayOutputStream();
         ObjectOutputStream out = new ObjectOutputStream(bout);
         long start = System.currentTimeMillis();
         out.writeObject(date);
         out.close();
         saveTime += System.currentTimeMillis() - start;
         size = bout.size();
         ByteArrayInputStream bin = new ByteArrayInputStream(bout.toByteArray());
         ObjectInputStream in = new ObjectInputStream(bin);
         start = System.currentTimeMillis();
         in.readObject();
         in.close();
         loadTime += System.currentTimeMillis() - start;
      }
      System.out.println(date.getClass().getName() + " " + size + "bytes save: " + (saveTime / iterations) + "ms load: " + (loadTime / iterations) + "ms");
   }

}
