/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.serialization;

import java.io.Serializable;

/**
 * LinkedList with very inefficient Serialization.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class LinkedList1 implements Serializable, LinkedList {

   private Entry head;
   private int size;

   public LinkedList1() {
      head = new Entry();
   }

   public void add(String value) {
      Entry e = head;
      while (e.getNext() != null) {
         e = e.getNext();
      }
      Entry newEntry = new Entry();
      newEntry.setValue(value);
      e.setNext(newEntry);
      size++;
   }

   public String get(int index) {
      if (index >= size) throw new IndexOutOfBoundsException();
      Entry e = head.getNext();
      for (int i = 0; i < index; i++) {
         e = e.getNext();
      }
      return e.getValue();
   }

   public String remove(int index) {
      if (index >= size) throw new IndexOutOfBoundsException();
      Entry e = head.getNext();
      for (int i = 0; i < index; i++) {
         e = e.getNext();
      }
      e.getPrev().setNext(e.getNext());
      if (e.getNext() != null)
         e.getNext().setPrev(e.getPrev());
      return e.getValue();
   }

   private static class Entry implements Serializable {
      private Entry next;
      private Entry prev;
      private String value;

      public String getValue() {
         return value;
      }

      public void setValue(String value) {
         this.value = value;
      }

      public Entry getNext() {
         return next;
      }

      public void setNext(Entry next) {
         this.next = next;
      }

      public Entry getPrev() {
         return prev;
      }

      public void setPrev(Entry prev) {
         this.prev = prev;
      }
   }
}
