/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.text.ParseException;
import java.util.*;

/**
 * Parses a date with different parser classes.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class DateParseDemo {
    private static DateFormat simpleDateFormat = new SimpleDateFormat("dd MMM yyyy HH:mm:ss,SSS");
    private static DateFormat dateTimeDateFormat = new DateTimeDateFormat();
    private static DateFormat diffDateFormat = new DifferenceDateFormat();

    private static String[] fullDates = new String[1000];
    private static String[] diffDates = new String[1000];
    private static String[] dates = new String[1000];
    private static Date[] dateObjects = new Date[1000];

    public static void main(String[] args) throws ParseException {
        int iterations = 500;
        long start;
        long time;
        System.out.println("Building date strings...");
        for (int i=0; i<1000; i++) dateObjects[i] = new Date(System.currentTimeMillis() + i * 7);

        // shuffle dates
        List list = new ArrayList(Arrays.asList(dateObjects));
        Collections.shuffle(list);
        dateObjects = (Date[])list.toArray(dateObjects);

        for (int i=0; i<1000; i++) fullDates[i] = simpleDateFormat.format(dateObjects[i]);
        for (int i=0; i<1000; i++) dates[i] = dateObjects[i].toString();
        DateFormat ddf = new DifferenceDateFormat();
        for (int i=0; i<1000; i++) diffDates[i] = ddf.format(dateObjects[i]);
        System.out.println("Done");

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) date();
        time = System.currentTimeMillis() - start;
        System.out.println("date first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) date();
        time = System.currentTimeMillis() - start;
        System.out.println("date second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) simpleDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("simpleDateFormat first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) simpleDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("simpleDateFormat second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) dateTimeDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("dateTimeDateFormat first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) dateTimeDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("dateTimeDateFormat second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) diffDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("diffDateFormat first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) diffDateFormat();
        time = System.currentTimeMillis() - start;
        System.out.println("diffDateFormat second run: " + time);
    }

    public static void date() {
        for (int i=0; i<1000; i++) {
            Date date = new Date(Date.parse(dates[i]));
            //System.out.println(dateString);
        }
    }

    public static void dateTimeDateFormat() throws ParseException {
        for (int i=0; i<1000; i++) {
            Date date = dateTimeDateFormat.parse(fullDates[i]);
            //System.out.println(dateString);
        }
    }

    public static void simpleDateFormat() throws ParseException {
        for (int i=0; i<1000; i++) {
            Date date = simpleDateFormat.parse(fullDates[i]);
            //System.out.println(dateString);
        }
    }

    public static void diffDateFormat() throws ParseException {
        for (int i=0; i<1000; i++) {
            Date date = diffDateFormat.parse(diffDates[i]);
            //System.out.println(dateString);
        }
    }

}
