/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.text.NumberFormat;
import java.text.ParseException;

/**
 * Converts a String into an int with different parser classes.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class IntegerParseDemo {
    private static NumberFormat formatter = NumberFormat.getInstance();

    public static void main(String[] args) throws Exception {
        int iterations = 5000;
        long start;
        long time;

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseInteger();
        time = System.currentTimeMillis() - start;
        System.out.println("parseInteger first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseInteger();
        time = System.currentTimeMillis() - start;
        System.out.println("parseInteger second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseLong();
        time = System.currentTimeMillis() - start;
        System.out.println("parseLong first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseLong();
        time = System.currentTimeMillis() - start;
        System.out.println("parseLong second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) formatterParseInteger();
        time = System.currentTimeMillis() - start;
        System.out.println("formatterParseInteger first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) formatterParseInteger();
        time = System.currentTimeMillis() - start;
        System.out.println("formatterParseInteger second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseIntegerParser();
        time = System.currentTimeMillis() - start;
        System.out.println("parseIntegerParser first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) parseIntegerParser();
        time = System.currentTimeMillis() - start;
        System.out.println("parseIntegerParser second run: " + time);
    }

    public static void parseInteger() {
        for (int i=0; i<1000; i++) Integer.parseInt("123456");
    }
    public static void parseLong() {
        for (int i=0; i<1000; i++) Long.parseLong("123456");
    }
    public static void formatterParseInteger() throws ParseException {
        for (int i=0; i<1000; i++) formatter.parse("123456").intValue();
    }

    public static void parseIntegerParser() {
        for (int i=0; i<1000; i++) NumberParser.parseInt("123456");
    }
}
