/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.text.MessageFormat;
import java.util.Date;

/**
 * Creates a message in different ways.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class MessageFormatDemo {
    private static MessageFormat formatter = new MessageFormat("On {1,time, long} {1,date, long}, there was {2} on planet {0,number,integer}.");

    public static void main(String[] args) {
        int iterations = 50;
        long start;
        long time;

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) preFormattedMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("preFormattedMessage first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) preFormattedMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("preFormattedMessage second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) plainMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("plainMessage first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) plainMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("plainMessage second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) formattedMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("formattedMessage first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) formattedMessage();
        time = System.currentTimeMillis() - start;
        System.out.println("formattedMessage second run: " + time);
    }

    public static void formattedMessage() {
        for (int i=0; i<1000; i++) {
            String message = MessageFormat.format("On {1,time, long} {1,date, long}, there was {2} on planet {0,number,integer}.",
                    new Object[] {new Integer(1), new Date(), "a disturbance in the Force"});
            //System.out.println(message);
        }
    }

    public static void preFormattedMessage() {
        for (int i=0; i<1000; i++) {
            String message = formatter.format(new Object[] {new Integer(1), new Date(), "a disturbance in the Force"});
            //System.out.println(message);
        }
    }
    public static void plainMessage() {
        for (int i=0; i<1000; i++) {
            String what = "a disturbance in the Force";
            Date date = new Date();
            int number = 1;
            String message = "On " + date + ", there was " + what + " on planet " + number;
            //System.out.println(message);
        }
    }
}
