/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

/**
 * Class that demonstrates concatenations with Strings or StringBuffers.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class StringConcatDemo2 {

    public static void main(String[] args) {
        int iterations = 2000;
        long start;
        long time;

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringConcat();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringConcat first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringConcat();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringConcat second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringBuffer first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringBuffer second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithPresizedStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithPresizedStringBuffer first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithPresizedStringBuffer();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithPresizedStringBuffer second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringsAndPlus();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringsAndPlus first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) concatWithStringsAndPlus();
        time = System.currentTimeMillis() - start;
        System.out.println("concatWithStringsAndPlus second run: " + time);
    }

    private static String concatWithStringConcat() {
        String s = "aString";
        for (int i = 0; i < 10; i++) s = s.concat(s);
        return s;
    }

    private static String concatWithPresizedStringBuffer() {
        StringBuffer sb = new StringBuffer(7168);
        sb.append("aString");
        for (int i = 0; i < 10; i++) sb.append(sb.toString());
        return sb.toString();
    }

    private static String concatWithStringBuffer() {
        StringBuffer sb = new StringBuffer("aString");
        for (int i = 0; i < 10; i++) sb.append(sb.toString());
        return sb.toString();
    }

    private static String concatWithStringsAndPlus() {
        String s = "aString";
        for (int i = 0; i < 10; i++) s += s;
        return s;
    }

}
