/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

/**
 * Class that demonstrates {@link String#intern()}.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class StringInternDemo {

    private static String a = "0123456789";
    private static String b = new String("0123456789");
    private static String c = "9012345678";
    private static String d = "d";

    public static void main(String[] args) {
        for (int i = 10; i != 0; i--) testrun(2500000, i);
    }

    private static void testrun(int iterations, int innerIterations) {
        System.out.println("InnerIterations: " + innerIterations);
        long start;
        long time;
        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareSameStrings(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareSameStrings first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareSameStrings(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareSameStrings second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareEqualStrings(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareEqualStrings first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareEqualStrings(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareEqualStrings second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareEqualStringsWithIntern(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareEqualStringsWithIntern first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareEqualStringsWithIntern(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareEqualStringsWithIntern second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareUnequalStringsSameLength(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareUnequalStringsSameLength first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareUnequalStringsSameLength(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareUnequalStringsSameLength second run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareUnequalStringsDifferentLength(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareUnequalStringsDifferentLength first run: " + time);

        start = System.currentTimeMillis();
        for (int i = 0; i < iterations; i++) compareUnequalStringsDifferentLength(innerIterations);
        time = System.currentTimeMillis() - start;
        System.out.println("compareUnequalStringsDifferentLength second run: " + time);
    }

    private static boolean compareSameStrings(int innerIterations) {
        boolean result = false;
        for (int i = 0; i < innerIterations; i++) result &= a.equals(a);
        return result;
    }

    private static boolean compareEqualStrings(int innerIterations) {
        boolean result = false;
        for (int i = 0; i < innerIterations; i++) result &= a.equals(b);
        return result;
    }

    private static boolean compareUnequalStringsSameLength(int innerIterations) {
        boolean result = false;
        for (int i = 0; i < innerIterations; i++) result &= a.equals(c);
        return result;
    }

    private static boolean compareUnequalStringsDifferentLength(int innerIterations) {
        boolean result = false;
        for (int i = 0; i < innerIterations; i++) result &= a.equals(d);
        return result;
    }

    private static boolean compareEqualStringsWithIntern(int innerIterations) {
        boolean result = false;
        String c = b.intern();
        for (int i = 0; i < innerIterations; i++) result &= a.equals(c);
        return result;
    }

}
