/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.xml;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.traversal.DocumentTraversal;
import org.w3c.dom.traversal.NodeFilter;
import org.w3c.dom.traversal.NodeIterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;

/**
 * Emits the names of all tags using a NodeIterator. This will only work
 * with a parser that supports the DOM Level 2 Traversal and Range API, such
 * as Xerces-J 2.0.1
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class DOMNodeIteratorDemo {

   public static void main(String[] args) throws Exception {
      DocumentBuilderFactory builderFactory = DocumentBuilderFactory.newInstance();
      // turn defer-node-expansion off/on
      builderFactory.setAttribute("http://apache.org/xml/features/dom/defer-node-expansion", Boolean.FALSE);
      DocumentBuilder documentBuilder = builderFactory.newDocumentBuilder();
      Document document = documentBuilder.parse(new File(args[0]));
      System.out.println(document.getClass());
      long start = System.currentTimeMillis();
      DocumentTraversal traversable = (DocumentTraversal) document;
      NodeIterator iterator = traversable.createNodeIterator(document, NodeFilter.SHOW_ELEMENT, null, true);
      Element element;
      while ((element = (Element) iterator.nextNode()) != null) {
         element.getTagName();
      }
      System.out.println(System.currentTimeMillis() - start);
   }
}
