/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.classloading;

import javax.swing.*;
import java.awt.*;
import java.awt.image.ImageObserver;
import java.net.URL;

/**
 * Shows an undecorated frame with an image in the center of the screen.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class SplashScreen extends Frame {
   private Image image;
   private boolean imageIsLoaded;
   private boolean errorWhileLoadingImage;

   public SplashScreen(String title, URL imageURL) {
      super(title);
      setCursor(Cursor.WAIT_CURSOR);
      setUndecorated(true);
      image = Toolkit.getDefaultToolkit().createImage(imageURL);
      // register for notification
      image.getHeight(this);
      // wait until we have height and width
      waitForWidthAndHeight();
      if (errorWhileLoadingImage) {
         JOptionPane.showMessageDialog(this, "Failed to load image from " + imageURL, "Error", JOptionPane.ERROR_MESSAGE);
      } else {
         int imageWidth = image.getWidth(this);
         int imageHeight = image.getHeight(this);
         DisplayMode dm = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDisplayMode();
         setBounds((dm.getWidth() - imageWidth) / 2, (dm.getHeight() - imageHeight) / 2, imageWidth, imageHeight);
         setVisible(true);
      }
   }

   public void update(Graphics g) {
      // prevent flickering
      paint(g);
   }

   public void paint(Graphics g) {
      g.drawImage(image, 0, 0, this);
   }

   public synchronized boolean imageUpdate(Image img, int infoflags,
                                           int x, int y, int width, int height) {
      errorWhileLoadingImage = errorWhileLoadingImage || (infoflags & (ImageObserver.ABORT | ImageObserver.ERROR)) != 0;
      imageIsLoaded = errorWhileLoadingImage || imageIsLoaded || ((infoflags & ImageObserver.WIDTH) != 0 && (infoflags & ImageObserver.HEIGHT) != 0);
      if (imageIsLoaded) notifyAll();
      return super.imageUpdate(img, infoflags, x, y, width, height) && (imageIsLoaded);
   }

   public synchronized void waitForWidthAndHeight() {
      try {
         while (!imageIsLoaded) {
            wait();
         }
      } catch (InterruptedException ie) {
         // ignore
      }
   }
}
