/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * LRU cache that uses {@link java.util.LinkedHashMap}.
 * <p>
 * Needs JDK &gt;= 1.4.0
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class LRUCache extends LinkedHashMap implements Cache {
   private int capacity;
   private int hits;
   private int misses;

   /**
    * Creates a LRU cache with the given capacity.
    */
   public LRUCache(int capacity) {
      super((int) (capacity / 0.75f), 0.75f, true);
      this.capacity = capacity;
   }

   public Object get(Object key) {
      Object obj = super.get(key);
      if (obj == null)
         misses++;
      else
         hits++;
      return obj;
   }

   /**
    * Returns true if size() exceeds the capacity of this cache (not map).
    */
   protected boolean removeEldestEntry(Map.Entry eldest) {
      return size() > capacity;
   }

   /**
    * Capacity of this cache.
    */
   public int getCapacity() {
      return capacity;
   }

   /**
    * Returns a number between 0.0 and 1.0. 1.0 is a 100% hit rate.
    */
   public float getHitRatio() {
      return ((float) hits) / (float) (hits + misses);
   }

}
