/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import com.tagtraum.perf.util.HuffmanTree;

import java.io.FilterOutputStream;
import java.io.OutputStream;
import java.io.IOException;

/**
 * Implementation of a HuffmanOutputStream. <em>Note that this implementation is not
 * optimized.</em>
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 * @see HuffmanOutputStream
 * @see HuffmanTree
 */
public class HuffmanOutputStream extends FilterOutputStream {

   private static final int DEFAULT_BLOCKSIZE = 16 * 1024;
   private HuffmanTree huffman;
   private byte[] buffer;
   private int pos;

   public HuffmanOutputStream(OutputStream out) {
      this(out, DEFAULT_BLOCKSIZE);
   }

   public HuffmanOutputStream(OutputStream out, int blocksize) {
      super(out);
      this.huffman = new HuffmanTree();
      this.buffer = new byte[blocksize];
   }

   public void write(int b) throws IOException {
      if (pos == buffer.length) {
         flush();
      }
      buffer[pos] = (byte)b;
      pos++;
   }

   public void write(byte b[], int off, int len) throws IOException {
      int leftInBuffer = buffer.length-pos;
      if (len < leftInBuffer) {
         System.arraycopy(b, off, buffer, pos, len);
         pos += len;
      }
      else {
         System.arraycopy(b, off, buffer, pos, leftInBuffer);
         pos += leftInBuffer;
         flush();
         write(b, off + leftInBuffer, len-leftInBuffer);
      }
   }

   public void flush() throws IOException {
      huffman.encode(buffer, 0, pos, out);
      out.flush();
      pos = 0;
   }
}
