/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import java.io.*;

/**
 * {@link FileCache.Entry} object that merely wraps a file object.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
class LiveFile implements FileCache.Entry {
   private File file;

   /**
    * Initializes this entry with a file, that is read into a byte array.
    */
   public LiveFile(File file) throws IOException {
      this.file = file;
   }

   public InputStream getNewInputStream() throws IOException {
      return new FileInputStream(file);
   }

   public File getFile() {
      return file;
   }

   public long length() {
      return file.length();
   }

   public long lastModified() {
      return file.lastModified();
   }

   public void setTimeToLive(int timeToLive) {
      throw new UnsupportedOperationException();
   }

   public int getTimeToLive() {
      return -1;
   }

   public boolean isStale() {
      return false;
   }

}
