/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.servlet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.OutputStream;
import java.util.zip.GZIPOutputStream;

/**
 * Simple servlet that compresses output if possible.<p>
 * <em>Note: </em> You should not include other resources with a RequestDispatcher
 * as they will not be compressed and corrupt the stream.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 * @see GZIPFilter
 */
public abstract class CompressorServlet extends HttpServlet {

   public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
      OutputStream out = getOutputStream(request, response);
      try {
         writeToStream(request, response, out);
      }
      finally {
         out.close();
      }
   }

   public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
      OutputStream out = getOutputStream(request, response);
      try {
         writeToStream(request, response, out);
      }
      finally {
         out.close();
      }
   }

   private OutputStream getOutputStream(HttpServletRequest request, HttpServletResponse response) throws IOException {
      OutputStream out;
      if (acceptsGzip(request)) {
         response.setHeader("Content-Encoding", "gzip");
         out = new GZIPOutputStream(response.getOutputStream());
      } else {
         out = response.getOutputStream();
      }
      return out;
   }

   private boolean acceptsGzip(HttpServletRequest request) {
      String acceptEncoding = request.getHeader("Accept-Encoding");
      return acceptEncoding != null && acceptEncoding.indexOf("gzip") != -1;
   }

   public abstract void writeToStream(HttpServletRequest request, HttpServletResponse response, OutputStream out) throws IOException, ServletException;
}
