/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.io.Serializable;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Case insensitive set.
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class CaseInsensitiveSet implements Set, Serializable {
    private Set set;

    public CaseInsensitiveSet() {
        this.set = new HashSet();
    }

    // string specific methods

    public boolean add(String key) {
        if (key == null) return set.add(null);
        return set.add(key.toUpperCase());
    }

    public boolean remove(String key) {
        if (key == null) return set.remove(null);
        return set.remove(key.toUpperCase());
    }


    public boolean contains(String key) {
        if (key == null) return set.contains(null);
        return set.contains(key.toUpperCase());
    }

    // other methods

    public int size() {
        return set.size();
    }

    public boolean isEmpty() {
        return set.isEmpty();
    }

    public boolean add(Object key) {
        return add((String) key);
    }

    public boolean contains(Object key) {
        return contains((String) key);
    }

    public boolean addAll(Collection c) {
        return set.addAll(c);
    }

    public void clear() {
        set.clear();
    }

    public Iterator iterator() {
        return set.iterator();
    }

    public Object[] toArray() {
        return set.toArray();
    }

    public Object[] toArray(Object a[]) {
        return set.toArray(a);
    }

    public boolean remove(Object o) {
        return remove((String) o);
    }

    public boolean removeAll(Collection c) {
        return set.removeAll(c);
    }

    public boolean retainAll(Collection c) {
        return set.retainAll(c);
    }

    public boolean containsAll(Collection c) {
        return set.containsAll(c);
    }

    public int hashCode() {
        return set.hashCode();
    }

    public boolean equals(Object obj) {
        return set.equals(obj);
    }

    public String toString() {
        return set.toString();
    }
}


