/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.text.DateFormatSymbols;
import java.text.ParsePosition;
import java.text.FieldPosition;
import java.util.*;

/**
 * Parses dates in the format "dd MMM YYYY HH:mm:ss,SSS" for example, "06 Nov 1994 08:49:37,459".
 *
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class DateTimeDateFormat extends AbsoluteTimeDateFormat {

    private Map monthsMap;
    private String lastString;
    private int lastOffset;
    private int lastYear;
    private int lastDay;
    private int lastMonth;

    public DateTimeDateFormat() {
        super();
        lastString = "";
        String[] shortMonths = new DateFormatSymbols().getShortMonths();
        monthsMap = new HashMap();
        for (int i=0; i<shortMonths.length; i++) monthsMap.put(shortMonths[i], new Integer(i));
    }

    public DateTimeDateFormat(TimeZone timeZone) {
        this();
        setCalendar(Calendar.getInstance(timeZone));
    }

    public StringBuffer format(Date date, StringBuffer toAppendTo,
                               FieldPosition fieldPosition) {
        // delegate to e.g. log4J version of this method
        return null;
    }

    protected void subParse(String s, ParsePosition p) {
        if (!getCachedDate(s, p)) {
            int offset = p.getIndex();
            parseDate(s, p);
            putCachedDate(s, offset);
        }
        super.subParse(s, p);
    }

    /**
     * Parses the date portion of the given String and sets its values in the current calendar object.
     */
    private void parseDate(String s, ParsePosition p) {
        calendar.set(Calendar.DAY_OF_MONTH, NumberParser.parseInt(s, p.getIndex(), 2));
        calendar.set(Calendar.YEAR, NumberParser.parseInt(s, p.getIndex() + 7, 4));
        calendar.set(Calendar.MONTH, ((Integer)monthsMap.get(s.substring(p.getIndex() + 3, p.getIndex() + 6))).intValue());
        p.setIndex(p.getIndex() + 12);
    }

    /**
     * Returns true if a cached date could be obtained and set in the current calendar object.
     */
    private boolean getCachedDate(String s, ParsePosition p) {
        if (s.regionMatches(p.getIndex(), lastString, lastOffset, 11)) {
            calendar.set(Calendar.DAY_OF_MONTH, lastDay);
            calendar.set(Calendar.YEAR, lastYear);
            calendar.set(Calendar.MONTH, lastMonth);
            p.setIndex(p.getIndex() + 12);
            return true;
        }
        return false;
    }

    /**
     * Associates the date portion of the current calendar object with the given string.
     */
    private void putCachedDate(String s, int offset) {
        lastDay = calendar.get(Calendar.DAY_OF_MONTH);
        lastYear = calendar.get(Calendar.YEAR);
        lastMonth = calendar.get(Calendar.MONTH);
        lastString = s;
        lastOffset = offset;
    }

}
