/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.threads;

/**
 *
 * Date: 18.03.2002
 * Time: 21:56:24
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
class RunnerThread implements Runnable {

   private static int count;
   // isolate runner related synchronization from other synchronizations
   private final Object mutex = new Object();
   private Runner runner;
   private Runnable runnable;
   private boolean running;
   private boolean working;
   private Thread thread;

   RunnerThread() {
      thread = new Thread("Runner-" + count++);
      thread.setDaemon(true);
   }

   public void work() {
      synchronized (mutex) {
         if (thread.isAlive()) {
            mutex.notify();
         } else {
            thread.start();
         }
      }
   }

   public void quit() {
      synchronized (mutex) {
         running = false;
         mutex.notify();
      }
   }

   public void run() {
      synchronized (mutex) {
         try {
            running = true;
            while (running) {
               working = true;
               runnable.run();
               working = false;
               // make sure we don't accidently create a memory leak
               runnable = null;
               if (runner.returnRunnerThread(this)) {
                  mutex.wait();
               } else {
                  quit();
               }
            }
         } catch (InterruptedException ie) {
            // should never happen
            ie.printStackTrace();
            quit();
         }
      }
   }

   void setRunnable(Runnable runnable) {
      this.runnable = runnable;
   }

   void setRunner(Runner runner) {
      this.runner = runner;
   }

   public boolean isAlive() {
      return thread.isAlive();
   }

   public String toString() {
      StringBuffer sb = new StringBuffer();
      sb.append(thread.toString());
      sb.append(" [working ");
      sb.append(working);
      sb.append(" on ");
      sb.append(runnable);
      sb.append("]");
      return sb.toString();
   }
}
