/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.Random;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestMatrixAlgorithms extends TestCase {

   public TestMatrixAlgorithms(String name) {
      super(name);
   }

   public void testSimpleConventional() {
      int[][] a = {{1,-2}, {3,4}};
      int[][] b = {{0,-1}, {-3,2}};
      int[][] c = MatrixOperations.conventionalMultiply(a, b);
      assertEquals(c[0][0], 6);
      assertEquals(c[0][1], -5);
      assertEquals(c[1][0], -12);
      assertEquals(c[1][1], 5);
   }

   public void testBig() {
      int[][] a = new int[128][128];
      int[][] b = new int[128][128];
      Random rnd = new Random();
      for (int i=0; i<a.length; i++) {
         for (int j=0; j<a.length; j++) {
            a[i][j] = rnd.nextInt(10)-5;
            b[i][j] = rnd.nextInt(10)-5;
         }
      }
      long start = System.currentTimeMillis();
      int[][] conventional = MatrixOperations.multiply(a, b);
      System.out.println(System.currentTimeMillis() - start);
   }

   public void testAdd() {
      int[][] a = {{1,-2, 1, 4}, {2, 7, 4, 1}, {1, 7, 4, 2}, {2, -4, 4, 5}};
      int[][] b = {{4,-2, 2, 5}, {3, -4, 6, 2}, {7, 5, 4, -1}, {9, 4, -4, 1}};
      int[][] c = MatrixOperations.add(a, b);
      assertEquals(c[0][0], 5);
      assertEquals(c[0][1], -4);
      assertEquals(c[0][2], 3);
      assertEquals(c[0][3], 9);
   }

   public void testSubtract() {
      int[][] a = {{1,-2, 1, 4}, {2, 7, 4, 1}, {1, 7, 4, 2}, {2, -4, 4, 5}};
      int[][] b = {{4,-2, 2, 5}, {3, -4, 6, 2}, {7, 5, 4, -1}, {9, 4, -4, 1}};
      int[][] c = MatrixOperations.subtract(a, b);
      assertEquals(c[0][0], -3);
      assertEquals(c[0][1], 0);
      assertEquals(c[0][2], -1);
      assertEquals(c[0][3], -1);
   }

   public static TestSuite suite() {
      return new TestSuite(TestMatrixAlgorithms.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
