/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.datastructures;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.util.HashMap;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestRandomMapCache extends TestCase {

   public TestRandomMapCache(String name) {
      super(name);
   }

   public void testCapacity() {
      RandomMapCache rmc = new RandomMapCache(new HashMap(), 100);
      assertEquals(128, rmc.getCapacity());
   }

   public void testGet() {
      RandomMapCache rmc = new RandomMapCache(new HashMap(), 100);
      assertNull(rmc.get("Hello"));
   }

   public void testPut() {
      RandomMapCache rmc = new RandomMapCache(new HashMap(), 100);
      assertNull(rmc.put("Hey", "You"));
   }

   public void testPutGet() {
      RandomMapCache rmc = new RandomMapCache(new HashMap(), 100);
      assertNull(rmc.put("Hey", "You"));
      assertEquals(rmc.get("Hey"), "You");
   }

   public void testPutPutNullGet() {
      RandomMapCache rmc = new RandomMapCache(new HashMap(), 100);
      assertNull(rmc.put("Hey", "You"));
      assertEquals(rmc.put("Hey", null), "You");
      assertNull(rmc.get("Hey"));
   }

   public static TestSuite suite() {
      return new TestSuite(TestRandomMapCache.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
