/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.*;


/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: TestFastStreamWriter.java,v 1.3 2001/10/28 12:26:23 Hendrik Exp $
 */
public class TestFastStreamWriter extends TestCase {

   public void testValidConversion() throws IOException {
      String testString = "ASCII-String";
      ByteArrayOutputStream stream = new ByteArrayOutputStream();
      Writer ascii = new FastStreamWriter(stream, "ASCII");
      ascii.write(testString);
      ascii.close();
      ByteArrayInputStream instream = new ByteArrayInputStream(stream.toByteArray());
      InputStreamReader in = new InputStreamReader(instream, "ASCII");
      int c;
      StringBuffer sb = new StringBuffer();
      while ((c = in.read()) != -1) {
         sb.append((char) c);
      }
      assertTrue(testString.equals(sb.toString()));
   }

   public void testUnicodeConversion() throws IOException {
      StringBuffer testStringBuffer = new StringBuffer();
      for (char i = 256; i < 1024; i++) {
         testStringBuffer.append(i);
      }
      String testString = testStringBuffer.toString();
      ByteArrayOutputStream stream = new ByteArrayOutputStream();
      Writer ascii = new FastStreamWriter(stream, "UTF-8");
      ascii.write(testString);
      ascii.close();
      ByteArrayInputStream instream = new ByteArrayInputStream(stream.toByteArray());
      InputStreamReader in = new InputStreamReader(instream, "UTF-8");
      int c;
      StringBuffer sb = new StringBuffer();
      while ((c = in.read()) != -1) {
         sb.append((char) c);
      }
      assertTrue(testString.equals(sb.toString()));
   }

   public void testInvalidConversion() throws IOException {
      String testString = "Non ASCII-String: ";
      ByteArrayOutputStream stream = new ByteArrayOutputStream();
      Writer ascii = new FastStreamWriter(stream, "ASCII");
      try {
         ascii.write(testString);
         fail("String should not have been converted.");
      } catch (IOException ioe) {
      }
      ascii.close();
   }


   public TestFastStreamWriter(String name) {
      super(name);
   }

   public static TestSuite suite() {
      return new TestSuite(TestFastStreamWriter.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }

}
