/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.io;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.EOFException;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestXtendedDataInputStream extends TestCase {

   public TestXtendedDataInputStream(String name) {
      super(name);
   }

   public void testBit() throws Exception {
      byte[] buf = new byte[]{0, (byte) 255};
      ByteArrayInputStream rawIn = new ByteArrayInputStream(buf);
      XtendedDataInputStream bitIn = new XtendedDataInputStream(rawIn);
      for (int i = 0; i < 2 * 8; i++) {
         boolean value = bitIn.readBit();
         assertEquals(value, i > 7);
      }
   }

   public void testBitEOF() throws Exception {
      byte[] buf = new byte[]{0, (byte) 255};
      ByteArrayInputStream rawIn = new ByteArrayInputStream(buf);
      XtendedDataInputStream bitIn = new XtendedDataInputStream(rawIn);
      try {
         for (int i = 0; i < 2 * 8 + 1; i++) {
            bitIn.readBit();
         }
         fail("Expected EOFException");
      } catch (EOFException eof) {
         // expected
      }
   }

   public void testHalfByte() throws Exception {
      byte[] buf = new byte[]{0, (byte) 255};
      ByteArrayInputStream rawIn = new ByteArrayInputStream(buf);
      XtendedDataInputStream halfByteIn = new XtendedDataInputStream(rawIn);
      assertEquals(0, halfByteIn.readHalfByte());
      assertEquals(0, halfByteIn.readHalfByte());
      assertEquals(0x0f, halfByteIn.readHalfByte());
      assertEquals(0x0f, halfByteIn.readHalfByte());
   }

   public void testWithOutputStream() throws Exception {
      ByteArrayOutputStream bout = new ByteArrayOutputStream();
      XtendedDataOutputStream hout = new XtendedDataOutputStream(bout);
      for (int i = 0; i < 13; i++) {
         hout.writeHalfByte(i);
      }
      hout.flush();
      ByteArrayInputStream rawIn = new ByteArrayInputStream(bout.toByteArray());
      XtendedDataInputStream halfByteIn = new XtendedDataInputStream(rawIn);
      for (int i = 0; i < 13; i++) {
         assertEquals(i, halfByteIn.readHalfByte());
      }
   }

   public void testHalfByteEOF() throws Exception {
      byte[] buf = new byte[]{0, (byte) 255};
      ByteArrayInputStream rawIn = new ByteArrayInputStream(buf);
      XtendedDataInputStream halfByteIn = new XtendedDataInputStream(rawIn);
      for (int i = 0; i < 2 * 2; i++) {
         halfByteIn.readHalfByte();
      }
      assertTrue(halfByteIn.readHalfByte() == -1);
   }


   public static TestSuite suite() {
      return new TestSuite(TestXtendedDataInputStream.class);
   }

   public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
   }
}
