/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestDifferenceDateFormat extends TestCase {

    public TestDifferenceDateFormat(String name) {
        super(name);
    }

    public void testFormat() {
        DateFormat formatter = new DifferenceDateFormat();
        Date now = new Date();
        Date later = new Date(now.getTime() + 1000);
        Date veryEarly = new Date(0);
        assertEquals(formatter.format(now), Long.toString(now.getTime()));
        assertEquals(formatter.format(later), "1000");
        assertEquals(formatter.format(veryEarly), Long.toString(-now.getTime()));
    }

    public void testParse() throws ParseException {
        DateFormat formatter = new DifferenceDateFormat();
        Date now = new Date();
        Date later = new Date(now.getTime() + 1000);
        Date veryEarly = new Date(0);
        assertEquals(formatter.parse(Long.toString(now.getTime())), now);
        assertEquals(formatter.parse("1000"), later);
        assertEquals(formatter.parse(Long.toString(-now.getTime())), veryEarly);
        formatter = new DifferenceDateFormat();
        assertEquals(formatter.parse("-1"), new Date(-1));
    }

    public void testParseError() {
        DateFormat formatter = new DifferenceDateFormat();
        try {
            formatter.parse("-a");
            fail("Expected ParseException.");
        }
        catch (ParseException pe) {
            // expected
        }
        try {
            formatter.parse("a23");
            fail("Expected ParseException.");
        }
        catch (ParseException pe) {
            // expected
        }
    }

    public static TestSuite suite() {
        return new TestSuite(TestDifferenceDateFormat.class);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }
}
