/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import java.util.Random;
import java.util.StringTokenizer;

import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestFastStringTokenizer extends TestCase {

    public TestFastStringTokenizer(String name) {
        super(name);
    }

    public void testMultipleDelims() {
        FastStringTokenizer st = new FastStringTokenizer(" t   ttt   t");
        assertTrue(st.countTokens() == 3);
        assertTrue(st.hasMoreTokens());
        assertTrue(st.nextToken().equals("t"));
        assertTrue(st.countTokens() == 2);
        assertTrue(st.hasMoreTokens());
        assertTrue(st.nextToken().equals("ttt"));
        assertTrue(st.countTokens() == 1);
        assertTrue(st.hasMoreTokens());
        assertTrue(st.nextToken().equals("t"));
        assertTrue(st.countTokens() == 0);
        assertTrue(!st.hasMoreTokens());
    }

    public void testLeadingDelim() {
        FastStringTokenizer st = new FastStringTokenizer(" t");
        assertTrue(st.countTokens() == 1);
        assertTrue(st.hasMoreTokens());
        assertTrue(st.nextToken().equals("t"));
        assertTrue(st.countTokens() == 0);
        assertTrue(!st.hasMoreTokens());
    }

    public void testTrailingDelim() {
        FastStringTokenizer st = new FastStringTokenizer("t  ");
        assertTrue(st.countTokens() == 1);
        assertTrue(st.hasMoreTokens());
        assertTrue(st.nextToken().equals("t"));
        assertTrue(st.countTokens() == 0);
        assertTrue(!st.hasMoreTokens());
    }


    public static TestSuite suite() {
        return new TestSuite(TestFastStringTokenizer.class);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }

}
