/*

Copyright (c) 2002 Hendrik Schreiber <hs@tagtraum.com>.

LGPL
====

This is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This software is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

For LGPL see <http://www.fsf.org/copyleft/lesser.txt>

For more information about this software see
<http://www.tagtraum.com/performance/>

*/
package com.tagtraum.perf.strings;

import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * @author <a href="mailto:hs@tagtraum.com">Hendrik Schreiber</a>
 * @version $Id: $
 */
public class TestNumberParser extends TestCase {

    public TestNumberParser(String name) {
        super(name);
    }

    public void testIntLetters() {
        try {
            NumberParser.parseInt("-1a");
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testLargeInt() {
        assertEquals(NumberParser.parseInt(Integer.toString(Integer.MAX_VALUE)), Integer.MAX_VALUE);
    }

    public void testSmallInt() {
        assertEquals(NumberParser.parseInt(Integer.toString(Integer.MIN_VALUE)), Integer.MIN_VALUE);
    }

    public void testTooSmallInt() {
        try {
            NumberParser.parseInt(Long.toString(((long) Integer.MIN_VALUE) - 1L));
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testTooLargeInt() {
        try {
            NumberParser.parseInt(Long.toString(((long) Integer.MAX_VALUE) + 1L));
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testLongLetters() {
        try {
            NumberParser.parseLong("-1a");
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testLargeLong() {
        assertEquals(NumberParser.parseLong(Long.toString(Long.MAX_VALUE)), Long.MAX_VALUE);
    }

    public void testSmallLong() {
        assertEquals(NumberParser.parseLong(Long.toString(Long.MIN_VALUE)), Long.MIN_VALUE);
    }

    public void testTooSmallLong() {
        try {
            NumberParser.parseLong(Long.toString(Long.MIN_VALUE) + "1");
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testTooLargeLong() {
        try {
            NumberParser.parseLong(Long.toString(Long.MAX_VALUE) + "1");
            fail("Expected NumberFormatException.");
        } catch (NumberFormatException nfe) {
            // expected
        }
    }

    public void testOffsetInt() {
        assertEquals(NumberParser.parseInt("a-1", 1, 2), -1);
        assertEquals(NumberParser.parseInt("a-1".toCharArray(), 1, 2), -1);
    }

    public void testOffsetLong() {
        assertEquals(NumberParser.parseLong("a-1", 1, 2), -1);
        assertEquals(NumberParser.parseLong("a-1".toCharArray(), 1, 2), -1);
    }


    public static TestSuite suite() {
        return new TestSuite(TestNumberParser.class);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(suite());
    }
}
